import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

/**
 * @author Nicolas
 * The grid, a digital frontier...
 */
public class Grid {

	private ArrayList<ArrayList<String>> matrix = new ArrayList<ArrayList<String>>();
	
	/**
	 * Constructeur : initialise la grille
	 */
	public Grid() {
		matrix.add( new ArrayList<String>());
		matrix.get(0).add("1");
		matrix.get(0).add("2");
		matrix.get(0).add("3");
		matrix.add(new ArrayList<String>());
		matrix.get(1).add("4");
		matrix.get(1).add("5");
		matrix.get(1).add("6");
		matrix.add(new ArrayList<String>());
		matrix.get(2).add("7");
		matrix.get(2).add("8");
		matrix.get(2).add("9");
	}
	
	/**
	 * Getter sur le contenu d'une cellule
	 * @param x le numéro de la cellule
	 * @return le contenu
	 */
	public String getCell(int x)
	{
		if(x>=0 && x<=9)
			return matrix.get((x-1)/3).get((x-1)%3);
		return(null);
	}
	
	/**
	 * Retourne une copie de la matrice
	 * @return la copie
	 */
	public ArrayList<ArrayList<String>> getAllCells()
	{
		return new ArrayList<ArrayList<String>>(matrix);
	}
	
	/**
	 * Renvoie vrai si la case ne contient pas de X ou de O
	 * @param le numéro de la cellule
	 * @return true si elle ne contient ni X ni O, false sinon.
	 */
	public boolean isEmptyCell(int x)
	{
		if(!getCell(x).equals("O") && !getCell(x).equals("X") && x>=1 && x<=9)
			return true;
		return false;
	}
	
	/**
	 * Vérifie si la partie est remportée
	 * @return
	 */
	public boolean isWon()
	{
		if(	getCell(1)==getCell(2) && getCell(2)==getCell(3) ||
			getCell(4)==getCell(5) && getCell(5)==getCell(6) ||
			getCell(7)==getCell(8) && getCell(8)==getCell(9) ||
			getCell(1)==getCell(4) && getCell(4)==getCell(7) ||
			getCell(2)==getCell(5) && getCell(5)==getCell(8) ||
			getCell(3)==getCell(6) && getCell(6)==getCell(9) ||
			getCell(1)==getCell(5) && getCell(5)==getCell(9) ||
			getCell(3)==getCell(5) && getCell(5)==getCell(7)	
			) return true;
		return false;
	}
	
	/**
	 * Ajoute une String à l'emplacement indiqué si c'est un chiffre ou une marque
	 * @param x emplacement
	 * @param s nouvelle valeur
	 */
	public void put(int x, String s)
	{
		Collection<String> collection = new ArrayList<String>(Arrays.asList(new String[] { "1", "2", "3", "4", "5", "6", "7", "8", "9", "X", "O" }));
		if(collection.contains(s))
			matrix.get((x-1)/3).set((x-1)%3,s);
	}
	
	/**
	 * @return une chaine représentant la grille
	 */
	public String toString()
	{
		return 	  "-------------\n| " + getCell(1) + " | " + getCell(2) + " | " + getCell(3) + " |\n"
				+ "-------------\n| " + getCell(4) + " | " + getCell(5) + " | " + getCell(6) + " |\n"
				+ "-------------\n| " + getCell(7) + " | " + getCell(8) + " | " + getCell(9) + " |\n"
				+ "-------------\n";
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Grid g = new Grid();
		Agent agent1 = new HumanAgent(g, "1", "X");
		Agent agent2 = new HumanAgent(g, "2", "O");
		
		int tour=0;
		while(!g.isWon())
		{
			System.out.println(g);
			if(tour==0)
				agent1.play();
			else agent2.play();
			tour = 1-tour;
		}
		System.out.println(g);
		System.out.println("Bravo ! C'est Fini !!");
	}

}
